/*
*  Name:         test-iterators.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Test for SQL iterators
*  Date:         $Date: 2001/10/21 20:17:41 $
*  Revision:     $Revision: 1.15 $
*
*  Copyright (C) 1999-2001  Francisco Vides Fernandez <pax@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/log.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>
#include <uesqlc/base_data_source.hh>
#include <uesqlc/iterator.hh>
#include <uesqlc/query.hh>

using mpcl::xstring;
using uesql::ComposeSchemaLocation;
using uesql::ComposeUserLogin;
using uesql::Quote;
using uesql::TSqlDataType;
using uesql::TSqlDate;
using uesql::TSqlInteger;
using uesql::TSqlReal;
using uesql::TSqlString;
using uesql::TIterator;
using uesql::TExceptionUESQL_Sqlerror;
using uesql::TQuery;


//
//  L O C A L   C L A S S E S
//

class TTestInstance
{

  public:

    TSqlDate      tAttrib0;
    TSqlInteger   tAttrib1;
    TSqlReal      tAttrib2;
    TSqlString    tAttrib3;

    
  public:

    TTestInstance() :
      tAttrib0()    ,
      tAttrib1()    ,
      tAttrib2()    ,
      tAttrib3()    {}


  public: 

    TSqlDataType& operator [] (unsigned int uiI)
    {
      switch (uiI)
      {
        case 0:
        {
          return tAttrib0;
          break;
        }
        case 1:
        {
          return tAttrib1;
          break;
        }
        case 2:
        {
          return tAttrib2;
          break;
        }
        case 3:
        {
          return tAttrib3;
          break;
        }
        default:        
        {
          throw mpcl::TExceptionEL_Constraint ("index out of range", __FILE__, __LINE__);
        }
      }
      return tAttrib0; // never reached
    }

    bool operator == (const TTestInstance& rktINSTANCE) const
    {
      return 
        ( tAttrib0 == rktINSTANCE.tAttrib0 ) &&
        ( tAttrib1 == rktINSTANCE.tAttrib1 ) &&
        ( tAttrib2 == rktINSTANCE.tAttrib2 ) &&
        ( tAttrib3 == rktINSTANCE.tAttrib3 ) ;
    }

};  // class TTestInstance


ostream& operator << (ostream& rtOSTREAM, TTestInstance& rtINSTANCE)
{

  for (register unsigned int I = 0; ( I < 4 ) ;++I)
  {
    rtOSTREAM << rtINSTANCE [I].get() << endl;
  }
  return rtOSTREAM;

}  // operator <<()


///  SQL back-end for \a TQuery.
class TDataSource : public uesql::TBaseDataSource<TTestInstance>
{

  protected:

    //
    //  S E L E C T O R S
    //

    void closeCursor (const mpcl::xstring& rkyCURSOR_NAME)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TExceptionUESQL_Sqlerror;
        close :rkyCURSOR_NAME;
      }
    }

    size_type declareCursor (const mpcl::xstring& rkyCURSOR_NAME)
    {
      TSqlInteger   iAux;

      extern "UESQL::default"
      {
        select count (*) into :iAux from instance;
        declare :rkyCURSOR_NAME cursor for 
          select attrib0, attrib1, attrib2, attrib3 from instance;
      }
      return (size_type) iAux;
    }

    void fetchAbsolute ( const mpcl::xstring& rkyCURSOR_NAME ,
                         TTestInstance&       rtINSTANCE     ,
                         size_type            zINDEX         )
    {
      extern "UESQL::default"
      {
        fetch absolute :uesql::TSqlInteger (zINDEX)
          from :rkyCURSOR_NAME
          into :rtINSTANCE.tAttrib0 ,
               :rtINSTANCE.tAttrib1 ,
               :rtINSTANCE.tAttrib2 ,
               :rtINSTANCE.tAttrib3 ;
      }
    }

    void fetchNext ( const mpcl::xstring& rkyCURSOR_NAME ,
                     TTestInstance&       rtINSTANCE     )
    {
      extern "UESQL::default"
      {
        fetch from :rkyCURSOR_NAME
          into :rtINSTANCE.tAttrib0 ,
               :rtINSTANCE.tAttrib1 ,
               :rtINSTANCE.tAttrib2 ,
               :rtINSTANCE.tAttrib3 ;
      }
    }

    void openCursor (const mpcl::xstring& rkyCURSOR_NAME)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TExceptionUESQL_Sqlerror;
        open :rkyCURSOR_NAME;
      }
    }

    void setErrorException (void)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TExceptionUESQL_Sqlerror;
      }
    }

};  // struct TDataSource


typedef uesql::TQuery<TTestInstance, TDataSource>  TTestQuery;


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::xstring _GetErrorMessage (void)
{

  using mpcl::xstring;

  return ( xstring (_tSqlstateDescriptor.getClass())    +
           ": "                                         +
           xstring (_tSqlstateDescriptor.getSubclass()) +
           " (" + uesql::SQLMESSAGE + ")"               );

}  // _GetErrorMessage()


static void _TestIteratorOperators (void)
{

  // Test ++I

  // Test I++

}   // _TestOperators()


int main (int argc, const char** argv)
{

  register int             I;
  mpcl::TConfigProcessor   tConfigProcessor (argc, argv);
  xstring                  yDbmsTestSchemaLocation;
  xstring                  yDbmsUserLogin;

  TEST_INIT ("Test for Cursor inside C++ Iterators");
  TEST_START_WATCH;
  try
  {
    tConfigProcessor.addOption ("dbms-server-name",   "o", "dbms");
    tConfigProcessor.addOption ("dbms-user-name",     "u", "test");
    tConfigProcessor.addOption ("dbms-user-password", "p", "test");
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor["dbms-user-name"].setUsageString ("");
    tConfigProcessor["dbms-user-password"].setUsageString ("");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::tLog.also (cerr);
    }
    yDbmsTestSchemaLocation = ComposeSchemaLocation ( tConfigProcessor ["dbms-server-name"].getValue() ,
                                                      "TEST_ITERATORS_SCHEMA"                          );
    yDbmsUserLogin          = ComposeUserLogin ( tConfigProcessor ["dbms-user-name"].getValue()     ,
                                                 tConfigProcessor ["dbms-user-password"].getValue() );
    TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
    TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition throw an exception
      --
      whenever sqlerror throw TExceptionUESQL_Sqlerror;
      connect to :Quote (yDbmsTestSchemaLocation)
        user :Quote (yDbmsUserLogin);
    }
    // <TODO comment="this will be the tests to run">
    _TestIteratorOperators ();
    // </TODO>    
    
    {
      // Iterate using a query
      TTestQuery             tTestQuery;
      TTestQuery::iterator   tTestIterator = tTestQuery.begin();

      // This one can test if an iterator's constructor returns a
      // valid one, being valid the data set
      if ( tTestIterator == tTestQuery.end() )
      {
        cout << "Alguna chorrada" << endl;
      }
      // Iterating a fixed number of times
      for (I = 0; I < 3 ;++I)
      {
        cout << *tTestIterator++;
      }
      // Iterating through the container as usual
      tTestIterator = tTestQuery.begin();
      for (; ( tTestIterator != tTestQuery.end() ) ;++tTestIterator)
      {
        cout << *tTestIterator;
      }
    }
  }
  catch (const uesql::TExceptionUESQL_Sqlerror& rktEXCEPTION)
  {
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
  }
  catch (const mpcl::TExceptionEL_Constraint& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    cerr << "Unhandled exception" << endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
